@@ -19,13 +19,15 @@ class Decoder(object):
             class_def_mapper = ClassDefMapper()
         self.class_def_mapper = class_def_mapper
 
-    def _getContext(self, input):
-        return DecoderContext(input, amf3=self.amf3, class_def_mapper=self.class_def_mapper);
+    def _getContext(self, input, amf3=None):
+        if amf3 is None:
+            amf3 = self.amf3
+        return DecoderContext(input, amf3=amf3, class_def_mapper=self.class_def_mapper)
 
-    def decode(self, val):
+    def decode(self, val, amf3=None):
         """Decode a string or file-like-object from AMF."""
-        return decode(self._getContext(val))
+        return decode(self._getContext(val, amf3))
 
-    def decode_packet(self, val):
+    def decode_packet(self, val, amf3=None):
         """Decode a string or file-like-object representing an AMF packet."""
-        return decode_packet(self._getContext(val))
+        return decode_packet(self._getContext(val, amf3))
