@@ -1,11 +1,58 @@
+import time
+
 from twisted.internet import defer, task, reactor
 from twisted.web import server
 from twisted.web.resource import Resource
 
-from amfast.remoting.channel import HttpChannel, Connection
+import amfast
 from amfast.class_def.as_types import AsNoProxy
-from amfast.remoting.endpoint import AmfEndpoint
-import amfast.remoting.flex_messages as messaging
+from channel import ChannelSet, HttpChannel
+from endpoint import AmfEndpoint
+import flex_messages as messaging
+import connection_manager as cm
+
+class TwistedChannelSet(ChannelSet):
+    """A ChannelSet optimized for use with Twisted."""
+
+    def render_POST(self, request):
+        channel_name = request.path[1:]
+        channel = self.getChannel(channel_name)
+        return channel.render_POST(request)
+
+    def scheduleClean(self):
+        reactor.callLater(self.clean_freq, self.clean)
+
+    def notifyConnections(self, topic, sub_topic):
+        reactor.callLater(0.1, self._notifyConnections, topic, sub_topic)
+
+    def _notifyConnections(self, topic, sub_topic):
+        for connection_id in self.subscription_manager.iterSubscribers(topic, sub_topic):
+            try:
+                connection = self.connection_manager.getConnection(connection_id, False)
+            except cm.NotConnectedError:
+                continue
+
+            if connection.notify_func is not None:
+                reactor.callLater(0.1, connection.notify_func)
+
+    def clean(self):
+        amfast.logger.debug("Cleaning connections.")
+
+        current_time = time.time()
+        deferreds = []
+        for connection_id in self.connection_manager.iterConnectionIds():
+            d = defer.Deferred()
+            d.addCallbacks(self.cleanConnection, callbackArgs=(current_time,))
+            reactor.callLater(0.3, d.callback, connection_id)
+
+        if len(deferreds) > 0:
+            def _scheduleClean(val):
+                self.scheduleClean()
+
+            deferredList = defer.DeferredList(deferreds)
+            deferredList.addCallback(self.scheduleClean)
+        else:
+            self.scheduleClean()
 
 class TwistedChannel(Resource, HttpChannel):
     """An AMF messaging channel that can be used with Twisted Web."""
@@ -24,8 +71,7 @@ class TwistedChannel(Resource, HttpChannel):
         timeout=1200, wait_interval=0):
 
         Resource.__init__(self)
-        HttpChannel.__init__(self, name, max_connections, endpoint,
-            timeout, wait_interval)
+        HttpChannel.__init__(self, name, max_connections, endpoint, wait_interval)
 
     def render_POST(self, request):
         """Process an incoming AMF packet."""
@@ -136,7 +182,7 @@ class TwistedChannel(Resource, HttpChannel):
         # Remove notify function if client drops connection.
         _connectionLost = request.connectionLost
         def _connection_lost(reason):
-            connection.setSessionAttr(connection.NOTIFY_KEY, False)
+            connection.unSetNotifyFunc()
             _connectionLost(reason)
             request.finish()
         request.connectionLost = _connection_lost
@@ -152,12 +198,12 @@ class TwistedChannel(Resource, HttpChannel):
 
             if timeout_call is not None and timeout_call.active():
                 timeout_call.cancel()
-
-            connection.setSessionAttr(connection.NOTIFY_KEY, False)
+            
+            connection.unSetNotifyFunc()
 
             # Get messages and add them
             # to the response message
-            messages = connection.poll()
+            messages = self.channel_set.subscription_manager.pollConnection(connection)
             
             if isinstance(packet.channel.endpoint, AmfEndpoint):
                 # Make sure messages are not encoded as an ArrayCollection
@@ -167,7 +213,7 @@ class TwistedChannel(Resource, HttpChannel):
             delattr(packet.response, self.MSG_NOT_COMPLETE)
             self.checkComplete(packet.response, request)
 
-        connection.setSessionAttr(connection.NOTIFY_KEY, _notify)
+        connection.setNotifyFunc(_notify)
 
         # Notify when wait_interval is reached
         if self.wait_interval > -1:
@@ -206,34 +252,35 @@ class StreamingTwistedChannel(TwistedChannel):
 
     def startStream(self, msg, request):
         """Get this stream rolling!"""
-
-        connection = self.channel_set.getConnection(msg.headers.get(msg.FLEX_CLIENT_ID_HEADER))
-        connection.setSessionAttr(self.TWISTED_REQUEST, request)
+        connection = self.channel_set.connection_manager.getConnection(msg.headers.get(msg.FLEX_CLIENT_ID_HEADER))
 
         # Remove notify function if client drops connection.
         _connectionLost = request.connectionLost
         def _connection_lost(reason):
-            connection.disconnect()
-            connection.setSessionAttr(connection.NOTIFY_KEY, False)
+            self.channel_set.disconnect(connection)
+            connection.unSetNotifyFunc()
             _connectionLost(reason)
         request.connectionLost = _connection_lost
 
         # This function gets called when a message is published.
         def _notify():
-            if connection.active is not True:
+            if connection.connected is False:
                 # Connection is disconnected
-                connection.setSessionAttr(connection.NOTIFY_KEY, False)
+                connection.unSetNotifyFunc()
+                msg = messaging.StreamingMessage.getDisconnectMsg()
+                request.write(messaging.StreamingMessage.prepareMsg(msg, self.endpoint))
+                request.finish()
                 return
 
-            while connection.hasMessages():
-                msg = connection.popMessage()
+            msgs = self.channel_set.subscription_manager.pollConnection(connection)
+            for msg in msgs:
                 request.write(messaging.StreamingMessage.prepareMsg(msg, self.endpoint))
-        connection.setSessionAttr(connection.NOTIFY_KEY, _notify)
+        connection.setNotifyFunc(_notify)
 
         # Acknowledge connection
         response = msg.acknowledge()
-        response.body = connection.flex_client_id
-        connection.publish(response)
+        response.body = connection.id
+        self.sendMsg(request, response)
 
         self.startBeat(connection, request)
 
@@ -242,27 +289,15 @@ class StreamingTwistedChannel(TwistedChannel):
         request.write(messaging.StreamingMessage.prepareMsg(msg, self.endpoint))
 
     def stopStream(self, msg, request):
-        connection = self.channel_set.getConnection(msg.headers.get(msg.FLEX_CLIENT_ID_HEADER))
-        connection.disconnect()
-
-    def disconnect(self, connection):
-        """Close response."""
-        TwistedChannel.disconnect(self, connection)
-
-        if not connection.hasSessionAttr(self.TWISTED_REQUEST):
-            return
-
-        request = connection.getSessionAttr(self.TWISTED_REQUEST)
-        msg = messaging.StreamingMessage.getDisconnectMsg()
-        request.write(messaging.StreamingMessage.prepareMsg(msg, self.endpoint))
-        request.finish()
+        connection = self.channel_set.connection_manager.getConnection(msg.headers.get(msg.FLEX_CLIENT_ID_HEADER))
+        self.channel_set.disconnect(connection)
 
     def startBeat(self, connection, request):
         # Send out heart beat.
         looper = task.LoopingCall(None)
         def _beat():
             """Keep calling this method as long as the connection is alive."""
-            if connection.active is not True:
+            if connection.connected is False:
                 looper.stop()
                 return
             
