@@ -4,14 +4,19 @@ import threading
 
 import amfast
 from amfast import AmFastError
-from amfast.remoting.channel import HttpChannel, Connection, ChannelError
+from amfast.remoting.channel import ChannelSet, HttpChannel, ChannelError
 import amfast.remoting.flex_messages as messaging
 
+class WsgiChannelSet(ChannelSet):
+    def __call__(self, environ, start_response):
+        channel_name = environ['PATH_INFO'][1:]
+        channel = self.getChannel(channel_name)
+        return channel(environ, start_response)
+
 class WsgiChannel(HttpChannel):
     """WSGI app channel."""
 
-    def __init__(self, name, max_connections=-1, endpoint=None,
-        timeout=1200, wait_interval=0):
+    def __init__(self, name, max_connections=-1, endpoint=None, wait_interval=0):
 
         if wait_interval < 0:
             # The only reliable way to detect
@@ -28,7 +33,7 @@ class WsgiChannel(HttpChannel):
             raise ChannelError('wait_interval < 0 is not supported by WsgiChannel')
             
         HttpChannel.__init__(self, name, max_connections=max_connections,
-            endpoint=endpoint, timeout=timeout, wait_interval=wait_interval)
+            endpoint=endpoint, wait_interval=wait_interval)
 
     def __call__(self, environ, start_response):
         if environ['REQUEST_METHOD'] != 'POST':
@@ -40,21 +45,21 @@ class WsgiChannel(HttpChannel):
         try:
             request_packet = self.decode(raw_request)
         except AmFastError, exc:
-            return self.badRequest(start_response, "AMF packet could not be decoded.")
+            return self.badRequest(start_response, self.getBadEncodingMsg())
         except (KeyboardInterrupt, SystemExit):
             raise
-        except:
-            amfast.log_exc()
-            return self.badServer(start_response, "AMF server error.")
+        except Exception, exc:
+            amfast.log_exc(exc)
+            return self.badServer(start_response, self.getBadServerMsg())
 
         try:
             content = self.invoke(request_packet)
             response = self.encode(content)
         except (KeyboardInterrupt, SystemExit):
             raise
-        except:
-            amfast.log_exc()
-            return self.badServer(start_response, "AMF server error.")
+        except Exception, exc:
+            amfast.log_exc(exc)
+            return self.badServer(start_response, self.getBadServerMsg())
 
         return self.getResponse(start_response, response)
 
@@ -67,7 +72,7 @@ class WsgiChannel(HttpChannel):
         return [response]
 
     def badMethod(self, start_response):
-        response = "405 Method Not Allowed\n\nAMF request must use 'POST' method."
+        response = self.getBadMethodMsg()
 
         start_response('405 Method Not Allowed', [
             ('Content-Type', 'text/plain'),
@@ -76,9 +81,7 @@ class WsgiChannel(HttpChannel):
 
         return [response]
 
-    def badRequest(self, start_response, msg):
-        response = "400 Bad Request\n\n%s" % msg
-
+    def badRequest(self, start_response, response):
         start_response('400 Bad Request', [
             ('Content-Type', 'text/plain'),
             ('Content-Length', str(len(response)))
@@ -86,9 +89,7 @@ class WsgiChannel(HttpChannel):
 
         return [response]
 
-    def badPage(self, start_response, msg):
-        response = "404 Not Found\n\n%s" % msg
-
+    def badPage(self, start_response, response):
         start_response('404 Not Found', [
             ('Content-Type', 'text/plain'),
             ('Content-Length', str(len(response)))
@@ -96,9 +97,7 @@ class WsgiChannel(HttpChannel):
 
         return [response]
 
-    def badServer(self, start_response, msg):
-        response = "500 Internal Server Error\n\n%s" % msg
-
+    def badServer(self, start_response, response):
         start_response('500 Internal Server Error', [
             ('Content-Type', 'text/plain'),
             ('Content-Length', str(len(response)))
@@ -109,10 +108,9 @@ class WsgiChannel(HttpChannel):
 class StreamingWsgiChannel(WsgiChannel):
     """WsgiChannel that opens a persistent connection with the client to serve messages."""
 
-    def __init__(self, name, max_connections=-1, endpoint=None,
-        timeout=1200, wait_interval=0, heart_interval=30):
-        WsgiChannel.__init__(self, name, max_connections, endpoint,
-            timeout, wait_interval)
+    def __init__(self, name, max_connections=-1, endpoint=None, wait_interval=0, heart_interval=30):
+        WsgiChannel.__init__(self, name, max_connections=max_connections,
+            endpoint=endpoint, wait_interval=wait_interval)
 
         self.heart_interval = heart_interval
 
@@ -132,7 +130,7 @@ class StreamingWsgiChannel(WsgiChannel):
             raise
         except Exception, exc:
             amfast.log_exc()
-            return self.badServer(start_response, "AMF server error.")
+            return self.badServer(start_response, self.getBadServerMsg())
 
         if msg.operation == msg.OPEN_COMMAND:
             return self.startStream(environ, start_response, msg)
@@ -140,18 +138,18 @@ class StreamingWsgiChannel(WsgiChannel):
         if msg.operation == msg.CLOSE_COMMAND:
             return self.stopStream(msg)
 
-        return self.badRequest(start_response, 'Streaming operation unknown: %s' % msg.operation)
+        return self.badRequest(start_response, self.getBadRequestMsg('Streaming operation unknown: %s' % msg.operation))
 
     def startStream(self, environ, start_response, msg):
         """Start streaming response."""
 
         try: 
-            connection = self.channel_set.getConnection(msg.headers.get(msg.FLEX_CLIENT_ID_HEADER))
+            connection = self.channel_set.connection_manager.getConnection(msg.headers.get(msg.FLEX_CLIENT_ID_HEADER))
         except (KeyboardInterrupt, SystemExit):
             raise
         except Exception, exc:
             amfast.log_exc()
-            return self.badServer(start_response, "AMF server error.")
+            return self.badServer(start_response, self.getBadServerMsg())
 
         write = start_response('200 OK', [
             ('Content-Type', self.CONTENT_TYPE)
@@ -160,7 +158,7 @@ class StreamingWsgiChannel(WsgiChannel):
         try:
             # Send acknowledge message
             response = msg.acknowledge()
-            response.body = connection.flex_client_id
+            response.body = connection.id
 
             try:
                 bytes = messaging.StreamingMessage.prepareMsg(response, self.endpoint)
@@ -173,15 +171,16 @@ class StreamingWsgiChannel(WsgiChannel):
 
             # Start heart beat
             timer = threading.Timer(self.heart_interval, self.beat, (connection, ))
+            timer.daemon = True
             timer.start()
 
             # Wait for new messages.
             event = threading.Event()
-            connection.setSessionAttr(connection.NOTIFY_KEY, event.set)
+            connection.setNotifyFunc(event.set)
             while True:
-                if connection.active is not True:
+
+                if connection.connected is False:
                     # Connection is no longer active
-                    connection.setSessionAttr(connection.NOTIFY_KEY, False)
                     msg = messaging.StreamingMessage.getDisconnectMsg()
                     try:
                         write(messaging.StreamingMessage.prepareMsg(msg, self.endpoint))
@@ -196,32 +195,35 @@ class StreamingWsgiChannel(WsgiChannel):
 
                 # Message has been published,
                 # or it's time for a heart beat
-                connection.setSessionAttr(connection.NOTIFY_KEY, False)
-                new_msg = connection.hasMessages()
-                if new_msg is True:
-                    # Send message
-                    while new_msg is True:
-                        msg = connection.popMessage()
 
-                        try:
-                            bytes = messaging.StreamingMessage.prepareMsg(msg, self.endpoint)
-                        except (KeyboardInterrupt, SystemExit):
-                            raise
-                        except Exception, exc:
-                            amfast.log_exc()
-                            connection.disconnect()
-                            break
+                # Remove notify_func so that
+                # New messages don't trigger event.
+                connection.unSetNotifyFunc()
 
-                        try:
-                            write(bytes)
-                        except (KeyboardInterrupt, SystemExit):
-                            raise
-                        except:
-                            # Client has disconnected
-                            connection.disconnect()
-                            return []
+                msgs = self.channel_set.subscription_manager.pollConnection(connection)
+                if len(msgs) > 0:
+                    while len(msgs) > 0:
+                        # Dispatch all messages to client
+                        for msg in msgs:
+                            try:
+                                bytes = messaging.StreamingMessage.prepareMsg(msg, self.endpoint)
+                            except (KeyboardInterrupt, SystemExit):
+                                raise
+                            except Exception, exc:
+                                amfast.log_exc()
+                                self.channel_set.disconnect(connection)
+                                break
 
-                        new_msg = connection.hasMessages()
+                            try:
+                                write(bytes)
+                            except (KeyboardInterrupt, SystemExit):
+                                raise
+                            except:
+                                # Client has disconnected
+                                self.channel_set.disconnect(connection)
+                                return []
+
+                        msgs = self.channel_set.subscription_manager.pollConnection(connection)
                 else:
                     # Send heart beat
                     try:
@@ -230,37 +232,34 @@ class StreamingWsgiChannel(WsgiChannel):
                         raise
                     except:
                         # Client has disconnected
-                        connection.disconnect()
+                        self.channel_set.disconnect(connection)
                         return []
 
                 # Create new event to trigger new messages or heart beats
                 event = threading.Event()
-                connection.setSessionAttr(connection.NOTIFY_KEY, event.set)
+                connection.setNotifyFunc(event.set)
         except (KeyboardInterrupt, SystemExit):
             raise
         except Exception, exc:
             amfast.log_exc()
-            connection.disconnect()
+            self.channel_set.disconnect(connection)
             return []
 
     def beat(self, connection):
         """Send a heart beat."""
-        if connection.active is not True:
-            # Client is disconnected
+        if hasattr(connection, "notify_func") and connection.notify_func is not None:
+            connection.notify_func()
+        else:
             return
 
-        notify_func = connection.getSessionAttr(connection.NOTIFY_KEY)
-        if notify_func is not False:
-            notify_func()
-
         # Create timer for next beat
         timer = threading.Timer(self.heart_interval, self.beat, (connection, ))
+        timer.daemon = True
         timer.start()
 
     def stopStream(self, msg):
         """Stop a streaming connection."""
         connection = self.channel_set.getConnection(msg.headers.get(msg.FLEX_CLIENT_ID_HEADER))
         connection.disconnect()
-        notify_func = connection.getSessionAttr(connect.NOTIFY_KEY)
-        if notify_func is not False:
-            notify_func()
+        if hasattr(connection, "notify_func") and connection.notify_func is not None:
+            connection.notify_func()
