@@ -1,16 +1,18 @@
 import unittest
 
-import amf3_encoder_test
-import amf0_encoder_test
-import amf3_decoder_test
-import amf0_decoder_test
-import round_trip_test
-import remoting_test
-import buffer_test
-import context_test
-import messaging_test
-
 def suite():
+    import amf3_encoder_test
+    import amf0_encoder_test
+    import amf3_decoder_test
+    import amf0_decoder_test
+    import round_trip_test
+    import remoting_test
+    import buffer_test
+    import context_test
+    import connection_test
+    import subscription_test
+    import messaging_test
+
     return unittest.TestSuite((
         amf3_decoder_test.suite(),
         amf3_encoder_test.suite(),
@@ -20,6 +22,8 @@ def suite():
         remoting_test.suite(),
         buffer_test.suite(),
         context_test.suite(),
+        connection_test.suite(),
+        subscription_test.suite(),
         messaging_test.suite()
     ))
 
