@@ -1,7 +1,10 @@
 # -*- coding: utf-8 -*-
 import unittest
 
-import amfast.decoder as decoder
+from StringIO import StringIO
+
+from amfast.context import DecoderContext
+import amfast.decode as decode
 import amfast.class_def as class_def
 
 class Amf3DecoderTestCase(unittest.TestCase):
@@ -13,13 +16,13 @@ class Amf3DecoderTestCase(unittest.TestCase):
         self.class_mapper = class_def.ClassDefMapper()
 
     def testNone(self):
-        self.assertEquals(None, decoder.decode('\x01', amf3=True))
+        self.assertEquals(None, decode.decode(DecoderContext('\x01', amf3=True)))
 
     def testFalse(self):
-        self.assertEquals(False, decoder.decode('\x02', amf3=True))
+        self.assertEquals(False, decode.decode(DecoderContext('\x02', amf3=True)))
 
     def testTrue(self):
-        self.assertEquals(True, decoder.decode('\x03', amf3=True))
+        self.assertEquals(True, decode.decode(DecoderContext('\x03', amf3=True)))
 
     def testInt(self):
         tests = {
@@ -40,7 +43,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         }
 
         for integer, encoding in tests.iteritems():
-            self.assertEquals(integer, decoder.decode(encoding, amf3=True))
+            self.assertEquals(integer, decode.decode(DecoderContext(encoding, amf3=True)))
 
     def testEncodeFloat(self):
         tests = {
@@ -49,7 +52,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         }
 
         for number, encoding in tests.iteritems():
-            self.assertEquals(number, decoder.decode(encoding, amf3=True))
+            self.assertEquals(number, decode.decode(DecoderContext(encoding, amf3=True)))
 
     def testEncodeLong(self):
         tests = {
@@ -59,7 +62,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         }
 
         for number, encoding in tests.iteritems():
-            self.assertEquals(number, decoder.decode(encoding, amf3=True))
+            self.assertEquals(number, decode.decode(DecoderContext(encoding, amf3=True)))
 
     def testUnicode(self):
         tests = {
@@ -69,7 +72,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         }
 
         for string, encoding in tests.iteritems():
-            self.assertEquals(string, decoder.decode(encoding, amf3=True))
+            self.assertEquals(string, decode.decode(DecoderContext(encoding, amf3=True)))
 
     def testUnicodeRefs(self):
         test = ['hello', 'hello', 'hello', 'hello']
@@ -80,7 +83,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x06\x00' #array element 3 (reference to hello)
         encoded += '\x06\x00' #array element 4 (reference to hello)
 
-        results = decoder.decode(encoded, amf3=True)
+        results = decode.decode(DecoderContext(encoded, amf3=True))
         for result in results:
             self.assertEquals('hello', result)
 
@@ -91,7 +94,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x04\x02' #array element 3
         encoded += '\x04\x03' #array element 4
 
-        result = decoder.decode(encoded, amf3=True)
+        result = decode.decode(DecoderContext(encoded, amf3=True))
         for i, val in enumerate(result):
             self.assertEquals(i, val);
 
@@ -102,7 +105,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x81\x2B' # String header
         encoded += '<?xml version="1.0" ?><test>\n            <test_me>tester</test_me>\n           </test>' # encoded XML
 
-        result = decoder.decode(encoded, amf3=True)
+        result = decode.decode(DecoderContext(encoded, amf3=True))
         self.assertEquals(xml.dom.minidom.Document, result.__class__)
 
     def testEncodeXmlRef(self):
@@ -111,7 +114,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x0B\x81\x2B<?xml version="1.0" ?><test>\n            <test_me>tester</test_me>\n           </test>' # array element 1 (encoded dom)
         encoded += '\x0B\x02' # array element 2 (reference to dom)
 
-        result = decoder.decode(encoded, amf3=True)
+        result = decode.decode(DecoderContext(encoded, amf3=True))
         self.assertEquals(xml.dom.minidom.Document, result[0].__class__)
         self.assertEquals(result[0], result[1])
 
@@ -121,7 +124,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x06\x09eggs' #value
         encoded += '\x01' # empty string terminator
 
-        result = decoder.decode(encoded, amf3=True)
+        result = decode.decode(DecoderContext(encoded, amf3=True))
         self.assertEquals('eggs', result['spam']);
 
     def testObjectProxy(self):
@@ -131,7 +134,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x06\x09eggs' #value
         encoded += '\x01' # empty string terminator
 
-        result = decoder.decode(encoded, amf3=True)
+        result = decode.decode(DecoderContext(encoded, amf3=True))
         self.assertEquals('eggs', result['spam']);
 
     def testDictRef(self):
@@ -139,7 +142,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x0A\x0B\x01\x09spam\x06\x09eggs\x01' #array element 1 (dict encoded)
         encoded += '\x0A\x02' # array element 2 (reference to dict)
 
-        result = decoder.decode(encoded, amf3=True)
+        result = decode.decode(DecoderContext(encoded, amf3=True))
         self.assertEquals('eggs', result[0]['spam'])
         self.assertEquals(result[0], result[1])
 
@@ -151,7 +154,11 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x04\x02' #array element 3
         encoded += '\x04\x03' #array element 4
 
-        result = decoder.decode(encoded, amf3=True)
+        result = decode.decode(DecoderContext(encoded, amf3=True))
+        for i in range(4):
+            self.assertEquals(i, result[i])
+
+        result = decode.decode(DecoderContext(StringIO(encoded), amf3=True))
         for i in range(4):
             self.assertEquals(i, result[i])
 
@@ -160,7 +167,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x0A\x01\x09\x09\x01\x04\x00\x04\x01\x04\x02\x04\x03' # array element 1 (test_tuple encoded)
         encoded += '\x0A\x04' # array element 2 (reference to test_tuple)
 
-        result = decoder.decode(encoded, amf3=True)
+        result = decode.decode(DecoderContext(encoded, amf3=True))
         self.assertEquals([].__class__, result.__class__)
         self.assertEquals(result[0], result[1])
         for i in range(4):
@@ -173,7 +180,8 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x0A\x13\x15alias.spam\x09spam\x06\x09eggs' # array element 1
         encoded += '\x0A\x01\x06\x07foo' # array element 2
 
-        result = decoder.decode(encoded, class_def_mapper=self.class_mapper, amf3=True)
+        result = decode.decode(DecoderContext(encoded,
+            class_def_mapper=self.class_mapper, amf3=True))
         self.class_mapper.unmapClass(self.Spam)
 
         self.assertEquals(result[0].__class__, result[1].__class__)
@@ -185,22 +193,32 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x09spam' # static attr definition
         encoded += '\x06\x09eggs' # static attrs
 
-        result = decoder.decode(encoded, class_def_mapper=self.class_mapper, amf3=True)
-        self.class_mapper.unmapClass(self.Spam)
-
+        result = decode.decode(DecoderContext(encoded,
+            class_def_mapper=self.class_mapper, amf3=True))
         self.assertEquals('eggs', result.spam)
 
+        result = decode.decode(DecoderContext(StringIO(encoded),
+            class_def_mapper=self.class_mapper, amf3=True))
+        self.assertEquals('eggs', result.spam)
+
+        self.class_mapper.unmapClass(self.Spam)
+
     def testDynamicObj(self):
         self.class_mapper.mapClass(class_def.DynamicClassDef(self.Spam, 'alias.spam', ()))
 
         encoded = '\x0A\x0B\x15alias.spam'
         encoded += '\x09spam\x06\x09eggs\x01' # dynamic attrs
 
-        result = decoder.decode(encoded, class_def_mapper=self.class_mapper, amf3=True)
-        self.class_mapper.unmapClass(self.Spam)
-
+        result = decode.decode(DecoderContext(encoded,
+            class_def_mapper=self.class_mapper, amf3=True))
         self.assertEquals('eggs', result.spam)
 
+        result = decode.decode(DecoderContext(StringIO(encoded),
+            class_def_mapper=self.class_mapper, amf3=True))
+        self.assertEquals('eggs', result.spam)
+
+        self.class_mapper.unmapClass(self.Spam)
+
     def testStaticDynamicObj(self):
         self.class_mapper.mapClass(class_def.DynamicClassDef(self.Spam, 'alias.spam', ('spam',)))
 
@@ -212,26 +230,34 @@ class Amf3DecoderTestCase(unittest.TestCase):
         encoded += '\x06\x09eggs' # static attrs
         encoded += '\x07ham\x06\x07foo\x01' #dynamic attrs
 
-        result = decoder.decode(encoded, class_def_mapper=self.class_mapper, amf3=True)
-        self.class_mapper.unmapClass(self.Spam)
-
+        result = decode.decode(DecoderContext(encoded,
+            class_def_mapper=self.class_mapper, amf3=True))
         self.assertEquals('eggs', result.spam)
         self.assertEquals('foo', result.ham)
 
+        result = decode.decode(DecoderContext(StringIO(encoded),
+            class_def_mapper=self.class_mapper, amf3=True))
+        self.assertEquals('eggs', result.spam)
+        self.assertEquals('foo', result.ham)
+
+        self.class_mapper.unmapClass(self.Spam)
+
     def testExternizeable(self):
         custom_encoding = '\x01\x02\x03\x04\x05'
 
-        class ExternClass(class_def.ExternizeableClassDef):
-            def readByteString(self, obj, buf):
+        class ExternClass(class_def.ExternClassDef):
+            def readExternal(ext_self, obj, context):
+                byte_string = context.buffer.read(5);
+                self.assertEquals(custom_encoding, byte_string)
                 obj.spam = 'eggs'
-                return 4
 
         self.class_mapper.mapClass(ExternClass(self.Spam, 'alias.spam', ('spam',)))
 
         encoded = '\x0A\x07\x15alias.spam'
         encoded += custom_encoding # raw bytes
 
-        result = decoder.decode(encoded, class_def_mapper=self.class_mapper, amf3=True)
+        result = decode.decode(DecoderContext(encoded,
+            class_def_mapper=self.class_mapper, amf3=True))
         self.class_mapper.unmapClass(self.Spam)
 
         self.assertEquals('eggs', result.spam)
@@ -239,7 +265,7 @@ class Amf3DecoderTestCase(unittest.TestCase):
     def testDate(self):
         import datetime
         encoded = '\x08\x01Bp+6!\x15\x80\x00'
-        result = decoder.decode(encoded, amf3=True)
+        result = decode.decode(DecoderContext(encoded, amf3=True))
         self.assertEquals(2005, result.year)
         self.assertEquals(3, result.month)
         self.assertEquals(18, result.day)
@@ -248,13 +274,16 @@ class Amf3DecoderTestCase(unittest.TestCase):
         self.assertEquals(31, result.second)
 
         encoded = '\x08\x01Bo%\xe2\xb2\x80\x00\x00'
-        result = decoder.decode(encoded, amf3=True)
+        result = decode.decode(DecoderContext(encoded, amf3=True))
         self.assertEquals(2003, result.year)
         self.assertEquals(12, result.month)
         self.assertEquals(1, result.day)
 
     def testUnkownByteRaisesException(self):
-        self.assertRaises(decoder.DecodeError, decoder.decode, '\x0D')
+        self.assertRaises(decode.DecodeError, decode.decode, DecoderContext('\x0D'))
+
+    def testBadArgRaisesException(self):
+        self.assertRaises(decode.DecodeError, decode.decode, 1)
 
 def suite():
     return unittest.TestLoader().loadTestsFromTestCase(Amf3DecoderTestCase)
